/**
 * @author Marlon Garcia
 * UTFPR - Campus Pato Branco - Licenciatura em Matemática
 * Classe Calculadora.java constrói uma tabela verdade
 * a partir de uma String em uma Matriz de caracteres.
 * 
 */
package com.example.tabelaverdade;

import java.util.ArrayList;
import java.util.List;

public class Calculadora {
	private String formula; // fórmula digitada pelo usuário
	private char[][] tabelaVerdade; // tabela verdade a ser retornada
	private int numeroProposicoes; // quantidade de proposições digitadas
	private int resultado = -1; // retorna a coluna resultado
	private List<Character> ordemProposicoes; // ordem em que as proposicoes foram digitadas
	private int quantidadeParenteses = 0; // marca a quantidade de parenteses digitados
	private int parentesesFim = 0; // localização do fim de parenteses
	private int parentesesInicio = 0; // localização do início dos parenteses
	
	// construtor com a fórmula como argumento
	public Calculadora(String formula) {
		this.formula = formula; // carrega a fórmula
		preencheColunas();
	}
	
	private void preencheColunas() {
		boolean p = false, q = false, r = false, s = false;
		numeroProposicoes = 0;
		ordemProposicoes = new ArrayList<Character>();
		
		// descobre ordem e quantidade das proposições
		for (int i = 0; i < formula.length(); i++) {
			if ((formula.charAt(i) == 'p') && (!p)) {
				numeroProposicoes++;
				ordemProposicoes.add('p');
				p = !p;
			} else if ((formula.charAt(i) == 'q') && (!q)) {
				numeroProposicoes++;
				ordemProposicoes.add('q');
				q = !q;
			} else if ((formula.charAt(i) == 'r') && (!r)) {
				numeroProposicoes++;
				ordemProposicoes.add('r');
				r = !r;
			} else if ((formula.charAt(i) == 's') && (!s)) {
				numeroProposicoes++;
				ordemProposicoes.add('s');
				s = !s;
			}
		}
		
		// calcula o número de linhas necessárias para a tabela verdade
		// baseado na quantidade de proposições da fórmula
		int linhas = (int) Math.pow(2, numeroProposicoes);
		
		// dimensiona a tabela verdade
		tabelaVerdade = new char[linhas + 1][formula.length()];
		
		// escreve a fórmula na primeira linha da tabela
		for (int i = 0; i < formula.length(); i++)
			tabelaVerdade[0][i] = formula.charAt(i);
		
		// preenche as colunas das proposições em sua devida ordem
		for (int n = 0; n < ordemProposicoes.size(); n++) {
			for (int i = 0; i < tabelaVerdade[0].length; i++) {
				if (tabelaVerdade[0][i] == ordemProposicoes.get(n))
					preencheColunaProposicao(i, n + 1);
			}
		}
		
		// calcula a quantidade de parenteses na fórmula
		for (int i = 0; i < tabelaVerdade[0].length; i++) {
			if (tabelaVerdade[0][i] == ')')
				quantidadeParenteses++;
		}
		
		// caso existam parenteses... cálculo
		if (quantidadeParenteses > 0) {
			// percorre a primeira linha buscando o primeiro ')'
			for (int i = 0; i < tabelaVerdade[0].length; i++) {
				if (tabelaVerdade[0][i] == ')') { // encontrado
					parentesesFim = i; // marca o primeiro fecha parenteses
					
					// retorna o loop buscando o respectivo '('
					for (int j = i; j >= 0; j--) {
						// encontrado o abre parenteses ainda não preenchido
						if ((tabelaVerdade[0][j] == '(') && (tabelaVerdade[1][j] != 'V') && 
								(tabelaVerdade[1][j] != 'F')) {
							parentesesInicio = j; // marca o abre parenteses
							
							if (parentesesFim - parentesesInicio == 2)
								resultado = parentesesFim - 1; // só há um caracter dentro
							
							// busca por negações dentro do bloco selecionado
							// de trás para frente
							for (int w = i; w >= j; w--) {
								if ((tabelaVerdade[0][w] == '-') && (tabelaVerdade[1][w] != 'V') && 
										(tabelaVerdade[1][w] != 'F')) {
									preencheColunaNegacao(w);
									resultado = w; // redundância da atribuição
								}
							}
							
							// busca por conjunções dentro do bloco selecinado
							for (int w = j; w <= i; w++) {
								if ((tabelaVerdade[0][w] == 'e') && (tabelaVerdade[1][w] != 'F') && 
										(tabelaVerdade[1][w] != 'V')) {
									preencheColunaConjuncao(w);
									resultado = w; // redundância da atribuição
								}
							}
							
							// busca por disjunções dentro do bloco selecionado
							for (int w = j; w <= i; w++) {
								if ((tabelaVerdade[0][w] == 'o') && (tabelaVerdade[1][w] != 'F') && 
										(tabelaVerdade[1][w] != 'V')) {
									preencheColunaDisjuncao(w);
									resultado = w; // redundância da atribuição
								}
							}
							
							// busca por condicionais dentro do bloco selecionado
							for (int w = j; w <= i; w++) {
								if ((tabelaVerdade[0][w] == 'c') && (tabelaVerdade[1][w] != 'F') && 
										(tabelaVerdade[1][w] != 'V')) {
									preencheColunaCondicional(w);
									resultado = w; // redundância da atribuição
								}
							}
							
							// busca por bicondicinais dentor do bloco selecionado
							for (int w = j; w <= i; w++) {
								if ((tabelaVerdade[0][w] == 'b') && (tabelaVerdade[1][w] != 'F') && 
										(tabelaVerdade[1][w] != 'V')) {
									preencheColunaBicondicional(w);
									resultado = w; // redundância da atribuição
								}
							}
							
							j = 0;
						}
					}
					
					// preenche a coluna dos parenteses para fins de cálculo
					preencheColunaParenteses(parentesesInicio, parentesesFim);
				}
				
				resultado = -1; // ao sair do bloco reseta a coluna resultado
			}
		}
		
		// busca por negações fora bloco de parenteses
		for (int i = tabelaVerdade[0].length - 1; i >= 0; i--) {
			if ((tabelaVerdade[0][i] == '-') && (tabelaVerdade[1][i] != 'V') && 
					(tabelaVerdade[1][i] != 'F')) {
				preencheColunaNegacao(i);
				resultado = i;
			}
		}
		
		// busca por conjunções fora de bloco
		for (int i = 0; i < tabelaVerdade[0].length; i++) {
			if ((tabelaVerdade[0][i] == 'e') && (tabelaVerdade[1][i] != 'V') && 
					(tabelaVerdade[1][i] != 'F')) {
				preencheColunaConjuncao(i);
				resultado = i;
			}
		}
		
		// busca por disjunções fora de bloco
		for (int i = 0; i < tabelaVerdade[0].length; i++) {
			if ((tabelaVerdade[0][i] == 'o') && (tabelaVerdade[1][i] != 'V') && 
					(tabelaVerdade[1][i] != 'F')) {
				preencheColunaDisjuncao(i);
				resultado = i;
			}
		}
		
		// busca por condicionais fora de bloco
		for (int i = 0; i < tabelaVerdade[0].length; i++) {
			if ((tabelaVerdade[0][i] == 'c') && (tabelaVerdade[1][i] != 'V') && 
					(tabelaVerdade[1][i] != 'F')) {
				preencheColunaCondicional(i);
				resultado = i;
			}
		}
		
		// busca por bicondicionais fora de bloco
		for (int i = 0; i < tabelaVerdade[0].length; i++) {
			if ((tabelaVerdade[0][i] == 'b') && (tabelaVerdade[1][i] != 'V') && 
					(tabelaVerdade[1][i] != 'F')) {
				preencheColunaBicondicional(i);
				resultado = i;
			}
		}
		
	}
	
	// método para calcular coluna de proposição, a partir de sua posição
	// e ordem.
	private void preencheColunaProposicao(int posicao, int ordemColuna) {
		// frequencia de alternância entre V e F
		int frequencia = (int) ((Math.pow(2, numeroProposicoes)) / 
				(Math.pow(2, ordemColuna)));
		// default V
		char valor = 'V';
		// indica fim de frequencia
		int flag = 0;
		
		// loop a partir da linha 2 preenchendo a coluna de proposição
		// alternando na frequencia entre V e F.
		for (int j = 1; j < tabelaVerdade.length; j++) {
			if (flag < frequencia)
				tabelaVerdade[j][posicao] = valor;
			else {
				if (valor == 'V')
					valor = 'F';
				else
					valor = 'V';
				tabelaVerdade[j][posicao] = valor;
				flag = 0;
			}
			flag++;
		}
	}
	
	// preenche a coluna dos parenteses temporariamente para fins de cálculo
	private void preencheColunaParenteses(int inicio, int fim) {
		// loop a partir da linha 2 preenchendo a coluna conforme resultado interno
		for (int j = 1; j < tabelaVerdade.length; j++) {
			tabelaVerdade[j][inicio] = tabelaVerdade[j][resultado];
			tabelaVerdade[j][fim] = tabelaVerdade[j][resultado];
		}
	}
	
	// método que analisa a formula para descobrir o operador anterior
	private int analisaAnteriores(char operador, int posicao) {
		int anterior = -1; // caso negativo
		
		// loop de trás para frente para analisar anteriores
		for (int i = posicao - 1; i >= 0; i--) {
			// se a posição é o primeiro caracter e uma negação, retorna
			if ((tabelaVerdade[0][i] == '-') && (i == 0)) {
				anterior = i;
				i = -1;
			} else if (tabelaVerdade[0][i] == '(') {
				i = -1; // não há anterior neste caso
			} else if (tabelaVerdade[0][i] == operador) { // achou anterior
				anterior = i; // retorna anterior
				i = -1;
			} else if (tabelaVerdade[0][i] == ')') { // caso fecha parenteses
				// descobre onde está o abre parenteses correspondente e pula
				// até a posição anterior para continuar a procura
				int cont = 1;
				for (int j = i - 1; cont > 0; j--) {
					if (tabelaVerdade[0][j] == ')')
						cont++;
					else if (tabelaVerdade[0][j] == '(')
						cont--;
					i--;
				}
			}
		}
		
		return anterior;
	}
	
	// método que analisa a fórmula para descobrir o operador posterior
	private int analisaPosteriores(char operador, int posicao) {
		int posterior = -1; // caso negativo
		
		// loop na primeira linha a partir da posição
		for (int i = posicao + 1; i < tabelaVerdade[0].length; i++) {
			if (tabelaVerdade[0][i] == ')') // não há posterior
				i = tabelaVerdade[0].length; // interrompe loop
			else if (tabelaVerdade[0][i] == '(') { // pula bloco de parenteses
				int cont = 1;
				for (int j = i + 1; cont > 0; j++) {
					if (tabelaVerdade[0][j] == '(')
						cont++;
					else if (tabelaVerdade[0][j] == ')')
						cont--;
					i = j;
				}
			} else if (tabelaVerdade[0][i] == operador) { // achou posterior
				posterior = i;
			}
		}
		
		return posterior;
	}
	
	// método para calcular e preencher as colunas com negacões
	private void preencheColunaNegacao(int posicao) {
		// loop a partir da segunda linha preenchendo a coluna na posição
		for (int j = 1; j < tabelaVerdade.length; j++) {
			if (tabelaVerdade[j][posicao + 1] == 'V')
				tabelaVerdade[j][posicao] = 'F';
			else
				tabelaVerdade[j][posicao] = 'V';
		}
		
		// atribui a atual coluna como resultado
		resultado = posicao;
	}
	
	// método para calcular e preencher as colunas de conjunção
	private void preencheColunaConjuncao(int posicao) {
		int anterior = posicao - 1; // supõe anterior
		int posterior = posicao + 1; // supõe posterior
		
		// Analisa anteriores
		if ((posicao > 2) && (tabelaVerdade[0][anterior - 1] == '-') && 
				(tabelaVerdade[0][anterior - 2] != 'e')) {
			anterior = anterior - 1; // anterior é uma negação
		} else if (analisaAnteriores('e', posicao) != -1) {
			anterior = analisaAnteriores('e', posicao); // conjunção encontrada
		}
		
		// calcula a coluna da conjunção
		for (int j = 1; j < tabelaVerdade.length; j++) {
			if ((tabelaVerdade[j][anterior] == 'V') && (tabelaVerdade[j][posterior] == 'V'))
				tabelaVerdade[j][posicao] = 'V'; // se e somente se ambos são verdade
			else
				tabelaVerdade[j][posicao] = 'F';
		}
		
		resultado = posicao; // atribui resultado
	}
	
	// método para calcular e preencher as colunas de disjunção
	private void preencheColunaDisjuncao(int posicao) {
		int anterior = posicao - 1; // supõe anterior
		int posterior = posicao + 1; // supõe posterior
		
		// Analisa anteriores
		if ((posicao > 2) && (tabelaVerdade[0][anterior - 1] == '-') && 
				(tabelaVerdade[0][anterior - 2] != 'e') && 
				(tabelaVerdade[0][anterior - 2] != 'o')) {
			anterior = anterior - 1; // anterior é uma negação
		} else if (analisaAnteriores('o', posicao) != -1)
			anterior = analisaAnteriores('o', posicao); // disjunção encontrada
		else if (analisaAnteriores('e', posicao) != -1)
			anterior = analisaAnteriores('e', posicao); // conjunção encontrada
		
		if (analisaPosteriores('e', posicao) != -1)
			posterior = analisaPosteriores('e', posicao); // conjunção encontrada
		
		// calcula a coluna da disjunção
		for (int j = 1; j < tabelaVerdade.length; j++) {
			if ((tabelaVerdade[j][anterior] == 'F') && (tabelaVerdade[j][posterior] == 'F'))
				tabelaVerdade[j][posicao] = 'F'; // se e somente se ambos são falsos
			else
				tabelaVerdade[j][posicao] = 'V';
		}
		
		resultado = posicao; // atribui resultado
	}
	
	// método para calcular e preencher as colunas de condicional
	private void preencheColunaCondicional(int posicao) {
		int anterior = posicao - 1; // supõe anterior
		int posterior = posicao + 1; // supõe posterior
		
		// Analisa anteriores
		if ((posicao > 2) && (tabelaVerdade[0][anterior - 1] == '-') && 
				(tabelaVerdade[0][anterior - 2] != 'e') && 
				(tabelaVerdade[0][anterior - 2] != 'o') && 
				(tabelaVerdade[0][anterior - 2] != 'c')) {
			anterior = anterior - 1; // anterior é uma negação
		} else if (analisaAnteriores('c', posicao) != -1) 
			anterior = analisaAnteriores('c', posicao); // condicional encontrada
		else if (analisaAnteriores('o', posicao) != -1) 
			anterior = analisaAnteriores('o', posicao); // disjunção encontrada
		else if (analisaAnteriores('e', posicao) != -1) 
			anterior = analisaAnteriores('e', posicao); // conjunção encontrada
		
		// Analisa posteriores
		if (analisaPosteriores('o', posicao) != -1)
			posterior = analisaPosteriores('o', posicao); // dijunção encontrada
		if (analisaPosteriores('e', posicao) != -1) 
			posterior = analisaPosteriores('e', posicao); // conjunção encontrada
		
		// calcula a coluna condicional
		for (int j = 1; j < tabelaVerdade.length; j++) {
			if ((tabelaVerdade[j][anterior] == 'V') && (tabelaVerdade[j][posterior] == 'F'))
				tabelaVerdade[j][posicao] = 'F'; // falso se e somente se o primeiro verdade e segundao falso
			else
				tabelaVerdade[j][posicao] = 'V';
		}
		
		resultado = posicao; // atribui resultdao à coluna
	}
	
	// método para calcular e preencher colunas bicondicinal
	private void preencheColunaBicondicional(int posicao) {
		int anterior = posicao - 1; // supõe anterior
		int posterior = posicao + 1; // supõe posterior
		
		// Analisa anteriores
		if ((posicao > 2) && (tabelaVerdade[0][anterior - 1] == '-') && 
				(tabelaVerdade[0][anterior - 2] != 'e') && 
				(tabelaVerdade[0][anterior - 2] != 'o') && 
				(tabelaVerdade[0][anterior - 2] != 'c') && 
				(tabelaVerdade[0][anterior - 2] != 'b')) {
			anterior = anterior - 1; // anterior é uma negação
		} else if (analisaAnteriores('b', posicao) != -1)
			anterior = analisaAnteriores('b', posicao); // bicondicional encontrada
		else if (analisaAnteriores('c', posicao) != -1) 
			anterior = analisaAnteriores('c', posicao); // condicional encontrada
		else if (analisaAnteriores('o', posicao) != -1)
			anterior = analisaAnteriores('o', posicao); // disjunção encontrada
		else if (analisaAnteriores('e', posicao) != -1) 
			anterior = analisaAnteriores('e', posicao); // conjunção encontrada
		
		// Analisa posteriores
		if (analisaPosteriores('c', posicao) != -1)
			posterior = analisaPosteriores('c', posicao); // condicional encontrada
		else if (analisaPosteriores('o', posicao) != -1) 
			posterior = analisaPosteriores('o', posicao); // dijunção encontrada
		else if (analisaPosteriores('e', posicao) != -1)
			posterior = analisaPosteriores('e', posicao); // conjunção encontrada
		
		// calcula a coluna bicondicional
		for (int j = 1; j < tabelaVerdade.length; j++) {
			if (tabelaVerdade[j][anterior] == tabelaVerdade[j][posterior])
				tabelaVerdade[j][posicao] = 'V'; // se e somente se ambos forem verdade
			else
				tabelaVerdade[j][posicao] = 'F';
		}
		
		resultado = posicao; // atribui coluna resultado
	}

	public char[][] getTabelaVerdade() {
		return tabelaVerdade;
	}

	public void setTabelaVerdade(char[][] tabelaVerdade) {
		this.tabelaVerdade = tabelaVerdade;
	}

	public int getResultado() {
		return resultado;
	}

	public void setResultado(int resultado) {
		this.resultado = resultado;
	}
	
}
