/**
 * @author Marlon Fernando Garcia
 * UTFPR - Campus Pato Branco - Licenciatura em Matemática
 * Classe Calculadora.java
 * Cria uma interface para calculadora
 */
package com.example.tabelaverdade;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.app.Fragment;
import android.graphics.Color;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

public class CalculadoraFragment extends Fragment {
	// retorna métodos implementados em MainActivity
	public interface CalculadoraFragmentListener {
		
		// usuário clica em Calcular
		public void onBotaoCalcularClicked(String formula);
		
	}
	
	private CalculadoraFragmentListener listener;
	
	private TextView textFormula;
	private Button botaoP;
	private Button botaoQ;
	private Button botaoR;
	private Button botaoS;
	private Button botaoNegacao;
	private Button botaoConjuncao;
	private Button botaoDisjuncao;
	private Button botaoCondicional;
	private Button botaoBicondicional;
	private Button botaoAbreParent;
	private Button botaoFechaParent;
	private Button botaoLimpar;
	private Button botaoCalcular;
	
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		listener = (CalculadoraFragmentListener) activity;
	}
	
	@Override
	public void onDetach() {
		super.onDetach();
		listener = null;
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		super.onCreateView(inflater, container, savedInstanceState);
		setRetainInstance(true);
		
		View view = inflater.inflate(R.layout.calculadora_fragment, container, false);
		
		// referencia os objetos
		textFormula = (TextView) view.findViewById(R.id.tvFormula);
		botaoP = (Button) view.findViewById(R.id.botaoP);
		botaoQ = (Button) view.findViewById(R.id.botaoQ);
		botaoR = (Button) view.findViewById(R.id.botaoR);
		botaoS = (Button) view.findViewById(R.id.botaoS);
		botaoNegacao = (Button) view.findViewById(R.id.botaoNegacao);
		botaoConjuncao = (Button) view.findViewById(R.id.botaoConjuncao);
		botaoDisjuncao = (Button) view.findViewById(R.id.botaoDisjuncao);
		botaoCondicional = (Button) view.findViewById(R.id.botaoCondicional);
		botaoBicondicional = (Button) view.findViewById(R.id.botaoBicondicional);
		botaoAbreParent = (Button) view.findViewById(R.id.botaoAbreParent);
		botaoFechaParent = (Button) view.findViewById(R.id.botaoFechaParent);
		botaoLimpar = (Button) view.findViewById(R.id.botaoLimpar);
		botaoCalcular = (Button) view.findViewById(R.id.botaoCalcular);
		
		botaoCalcular.setTextColor(Color.parseColor("#005900"));
		botaoLimpar.setTextColor(Color.parseColor("red"));
		
		botaoP.setOnClickListener(botaoPOnClickListener);
		botaoQ.setOnClickListener(botaoQOnClickListener);
		botaoR.setOnClickListener(botaoROnClickListener);
		botaoS.setOnClickListener(botaoSOnClickListener);
		botaoNegacao.setOnClickListener(botaoNegacaoOnClickListener);
		botaoConjuncao.setOnClickListener(botaoConjuncaoOnClickListener);
		botaoDisjuncao.setOnClickListener(botaoDisjuncaoOnClickListener);
		botaoCondicional.setOnClickListener(botaoCondicionalOnClickListener);
		botaoBicondicional.setOnClickListener(botaoBicondicionalOnClickListener);
		botaoAbreParent.setOnClickListener(botaoAbreParentOnClickListener);
		botaoFechaParent.setOnClickListener(botaoFechaParentOnClickListener);
		botaoLimpar.setOnClickListener(botaoLimparOnClickListener);
		botaoCalcular.setOnClickListener(botaoCalcularOnClickListener);
		
		return view;
	}
	
	// método para adicionar caracter na textFormula
	private void adicionaCaracter(String caracter) {
		textFormula.setText(textFormula.getText() + caracter);
	}
	
	OnClickListener botaoPOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.propP));
		}
	};
	
	OnClickListener botaoQOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.propQ));
		}
	};
	
	OnClickListener botaoROnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.propR));
		}
	};
	
	OnClickListener botaoSOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.propS));
		}
	};
	
	OnClickListener botaoNegacaoOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.negacao));
		}
	};
	
	OnClickListener botaoConjuncaoOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.conjuncao));
		}
	};
	
	OnClickListener botaoDisjuncaoOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.disjuncao));
		}
	};
	
	OnClickListener botaoCondicionalOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.condicional));
		}
	};
	
	OnClickListener botaoBicondicionalOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.bicondicional));
		}
	};
	
	OnClickListener botaoAbreParentOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.abreParentesis));
		}
	};
	
	OnClickListener botaoFechaParentOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			adicionaCaracter(getResources().getString(R.string.fechaParentesis));
		}
	};
	
	OnClickListener botaoLimparOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			if (textFormula.getText().length() > 0) {
				String temp = textFormula.getText().toString();
				String temp2 = temp.substring(0, temp.length() - 1);
				textFormula.setText(temp2);
			}
		}
	};
	
	OnClickListener botaoCalcularOnClickListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			// transforma para a linguagem da Calculadora
			String formulaTrabalhada = textFormula.getText().toString();
			char negacao = getResources().getString(R.string.negacao).charAt(0);
			char conjuncao = getResources().getString(R.string.conjuncao).charAt(0);
			char disjuncao = getResources().getString(R.string.disjuncao).charAt(0);
			char condicional = getResources().getString(R.string.condicional).charAt(0);
			char bicondicional = getResources().getString(R.string.bicondicional).charAt(0);
			formulaTrabalhada = formulaTrabalhada.replace(negacao, '-');
			formulaTrabalhada = formulaTrabalhada.replace(conjuncao, 'e');
			formulaTrabalhada = formulaTrabalhada.replace(disjuncao, 'o');
			formulaTrabalhada = formulaTrabalhada.replace(condicional, 'c');
			formulaTrabalhada = formulaTrabalhada.replace(bicondicional, 'b');
			
			// descobre se toda a fórmula está dentro de parenteses e retira
			// os parenteses desnecessários
			if (formulaTrabalhada.charAt(0) == '(') {
				int flag = 1;
				int posicao = 0;
				for (int i = 1; i < formulaTrabalhada.length(); i++) {
					if (formulaTrabalhada.charAt(i) == '(')
						flag++;
					else if (formulaTrabalhada.charAt(i) == ')')
						flag--;
					if (flag == 0) {
						posicao = i;
						i = formulaTrabalhada.length();
					}
				}
				if (posicao == formulaTrabalhada.length() - 1) 
					formulaTrabalhada = formulaTrabalhada.substring(1, 
							formulaTrabalhada.length() - 1);
			}
			
			if (!analisaParenteses(formulaTrabalhada))
				erroParentesesDialog.show(getFragmentManager(), "Erro parênteses");
			else if (!analisaPrimeiroUltimo(formulaTrabalhada))
				erroPrimeiroUltimoDialog.show(getFragmentManager(), "Operadores");
			else if (!analisaOperadoresJuntos(formulaTrabalhada))
				erroOperadoresJuntos.show(getFragmentManager(), "Operadores juntos");
			else if (!analisaProposicoesJuntas(formulaTrabalhada))
				erroProposicoesJuntas.show(getFragmentManager(), "Proposições juntas");
			else
				listener.onBotaoCalcularClicked(formulaTrabalhada);
		}
	};
	
	// certifica a quantidade de parenteses antes de enviar a Calculadora
	private boolean analisaParenteses(String formula) {
		int abreParenteses = 0;
		int fechaParenteses = 0;
		
		for (int i = 0; i < formula.length(); i++) {
			if (formula.charAt(i) == '(')
				abreParenteses++;
			else if (formula.charAt(i) == ')')
				fechaParenteses++;
		}
		
		if (abreParenteses == fechaParenteses)
			return true;
		else
			return false;
	}
	
	// certifica que o primeiro e o último caracter não sejam operadores
	private boolean analisaPrimeiroUltimo(String formula) {
		boolean flag = true;
		
		if (formula.charAt(0) == 'e' || formula.charAt(0) == 'o' || 
				formula.charAt(0) == 'c' || formula.charAt(0) == 'b')
			flag = false;
		else 
			flag = true;
		
		if (formula.charAt(formula.length() - 1) == '-' || 
				formula.charAt(formula.length() - 1) == 'e' || 
				formula.charAt(formula.length() - 1) == 'o' || 
				formula.charAt(formula.length() - 1) == 'c' || 
				formula.charAt(formula.length() - 1) == 'b')
			flag = false;
		else
			flag = true;
		
		return flag;
	}
	
	// certifica que não há operadores juntos
	private boolean analisaOperadoresJuntos(String formula) {
		for (int i = 0; i < formula.length() - 1; i++) {
			if (formula.charAt(i) == 'e' || formula.charAt(i) == 'o' || 
					formula.charAt(i) == 'c' || formula.charAt(i) == 'b') {
				if (formula.charAt(i + 1) == 'e' || formula.charAt(i + 1) == 'o' || 
						formula.charAt(i + 1) == 'c' || formula.charAt(i + 1) == 'b')
					return false;
			}
		}
		
		return true;
	}
	
	// certifica que não há proposições juntas
	private boolean analisaProposicoesJuntas(String formula) {
		for (int i = 0; i < formula.length() - 1; i++) {
			if (formula.charAt(i) == 'p' || formula.charAt(i) == 'q' || 
					formula.charAt(i) == 'r' || formula.charAt(i) == 's') {
				if (formula.charAt(i + 1) == 'p' || formula.charAt(i + 1) == 'q' || 
						formula.charAt(i + 1) == 'r' || formula.charAt(i + 1) == 's' ||
						formula.charAt(i + 1) == '-')
					return false;
			}
		}
		
		return true;
	}
	
	// diálogo de erro
	private DialogFragment erroParentesesDialog = new DialogFragment() {

		@Override
		public Dialog onCreateDialog(Bundle savedInstanceState) {
			AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
			builder.setMessage(R.string.erroNroParenteses);
			builder.setPositiveButton(R.string.ok, null);
			
			return builder.create();
		}
		
	};
	
	// diálogo de erro
	private DialogFragment erroPrimeiroUltimoDialog = new DialogFragment() {

		@Override
		public Dialog onCreateDialog(Bundle savedInstanceState) {
			AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
			builder.setMessage(R.string.erroUltimoOperador);
			builder.setPositiveButton(R.string.ok, null);
			
			return builder.create();
		}
		
	};
	
	// diálogo de erro
	private DialogFragment erroOperadoresJuntos = new DialogFragment() {

		@Override
		public Dialog onCreateDialog(Bundle savedInstanceState) {
			AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
			builder.setMessage(R.string.erroOperadoresJuntos);
			builder.setPositiveButton(R.string.ok, null);
			
			return builder.create();
		}
		
	};
	
	// diálogo de erro
	private DialogFragment erroProposicoesJuntas = new DialogFragment() {

		@Override
		public Dialog onCreateDialog(Bundle savedInstanceState) {
			AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
			builder.setMessage(R.string.erroProposicoesJuntas);
			builder.setPositiveButton(R.string.ok, null);
			
			return builder.create();
		}
		
	};
	
}
