/**
 * @author Marlon Fernando Garcia
 * UTFPR - Campus Pato Branco - Licenciatura em Matemática
 * Classe MainActivity.java
 * Classe que cria o container da atividade pricipal
 */
package com.example.tabelaverdade;

import com.example.tabelaverdade.AjudaFragmentLogica.AjudaFragmentLogicaListener;

import android.app.Activity;
import android.app.FragmentTransaction;
import android.os.Bundle;

public class MainActivity extends Activity implements MenuFragment.MenuFragmentListener, 
		CalculadoraFragment.CalculadoraFragmentListener, 
		TabelaFragment.TabelaFragmentListener, SobreFragment.SobreFragmentListener, 
		AjudaFragment.AjudaFragmentListener, AjudaFragmentLogicaListener {
	MenuFragment menuFragment; // fragment do menu
	public static String formulaTrabalhada; // dá acesso à formula
	public static char[][] tabelaVerdade; // dá acesso à tabela verdade
	public static int resultado; // dá acesso à coluna resultado
			
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main); // inicializa o container
		
		// retorna se a Activity foi restaurada
		if (savedInstanceState != null)
			return;
		
		// se o container foi criado e não recuperado
		if (findViewById(R.id.fragmentContainer) != null) {
			menuFragment = new MenuFragment(); // instancia um menu
			
			// utiliza um FragmentTransaction para criar o fragmento
			FragmentTransaction transaction = getFragmentManager().beginTransaction();
			transaction.add(R.id.fragmentContainer, menuFragment);
			transaction.commit();
		}
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		
		if (menuFragment == null)
			menuFragment = (MenuFragment) 
				getFragmentManager().findFragmentById(R.id.menuView);
	}

	@Override
	public void onBotaoCalcularClicked(String formula) {
		formulaTrabalhada = formula;
		// envia a fórmula para a Calculadora
		Calculadora calculadora = new Calculadora(formula);
		// retorna a coluna resultado e a tabela Verdade
		resultado = calculadora.getResultado();
		tabelaVerdade = calculadora.getTabelaVerdade();
		
		TabelaFragment tabelaFragment = new TabelaFragment();
		
		FragmentTransaction transaction = getFragmentManager().beginTransaction();
		transaction.replace(R.id.fragmentContainer, tabelaFragment);
		transaction.addToBackStack(null);
		transaction.commit();
	}

	@Override
	public void onBotaoCalculadoraClicked() {
		CalculadoraFragment calculadoraFragment = new CalculadoraFragment();
		
		FragmentTransaction transaction = getFragmentManager().beginTransaction();
		transaction.replace(R.id.fragmentContainer, calculadoraFragment);
		transaction.addToBackStack(null);
		transaction.commit();
	}

	@Override
	public void onBotaoAjudaClicked() {
		AjudaFragmentLogica ajudaFragment = new AjudaFragmentLogica();
		
		FragmentTransaction transaction = getFragmentManager().beginTransaction();
		transaction.replace(R.id.fragmentContainer, ajudaFragment);
		transaction.addToBackStack(null);
		transaction.commit();
	}

	@Override
	public void onBotaoSobreClicked() {
		SobreFragment sobreFragment = new SobreFragment();
		
		FragmentTransaction transaction = getFragmentManager().beginTransaction();
		transaction.replace(R.id.fragmentContainer, sobreFragment);
		transaction.addToBackStack(null);
		transaction.commit();
	}

	@Override
	public void onBotaoLogicaClicked() {
		AjudaFragmentLogica ajudaFragment = new AjudaFragmentLogica();
		
		FragmentTransaction transaction = getFragmentManager().beginTransaction();
		transaction.replace(R.id.fragmentContainer, ajudaFragment);
		transaction.addToBackStack(null);
		transaction.commit();
	}

	@Override
	public void onBotaoProposicaoClicked() {
		AjudaFragmentProposicao ajudaFragmentLogica = new AjudaFragmentProposicao();
		
		FragmentTransaction transaction = getFragmentManager().beginTransaction();
		transaction.replace(R.id.fragmentContainer, ajudaFragmentLogica);
		transaction.addToBackStack(null);
		transaction.commit();
	}

	@Override
	public void onBotaoTabelaVerdadeClicked() {
		AjudaFragmentTabela ajudaFragment = new AjudaFragmentTabela();
		
		FragmentTransaction transaction = getFragmentManager().beginTransaction();
		transaction.replace(R.id.fragmentContainer, ajudaFragment);
		transaction.addToBackStack(null);
		transaction.commit();
	}

	@Override
	public void onBotaoConectivosClicked() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onBotaoNegacaoClicked() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onBotaoConjuncaoClicked() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onBotaoDisjuncaoClicked() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onBotaoCondicionalClicked() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onBotaoBicondicionalClicked() {
		// TODO Auto-generated method stub
		
	}

}
